//--------------------------------------------
// Modified by: Sadegh Mohammadi
// http://sadmonew.blogfa.com
// 1391/03/28
//--------------------------------------------
//=============================================================================
#include "stm32f10x.h"
//=============================================================================
// Configuration
//=============================================================================
#define SSD1963_DATAPORT		GPIOB
#define SSD1963_DATAPORT_1	GPIOC
#define SSD1963_CTRLPORT		GPIOA

#define SSD1963_PIN_RD	(1 << 11)
#define SSD1963_PIN_WR	(1 << 12)
#define SSD1963_PIN_CS	(1 << 13)
#define SSD1963_PIN_A0	(1 << 14) // D/C
#define SSD1963_PIN_RST	(1 << 15)

//===================
#define TFT_FPS 60ULL
#define TFT_WIDTH				480ULL
#define TFT_HSYNC_BACK_PORCH	2ULL
#define TFT_HSYNC_FRONT_PORCH	2ULL
#define TFT_HSYNC_PULSE			41ULL
	
#define TFT_HEIGHT				272ULL
#define TFT_VSYNC_BACK_PORCH	2ULL
#define TFT_VSYNC_FRONT_PORCH	2ULL
#define TFT_VSYNC_PULSE			10ULL


#define	TFT_HSYNC_PERIOD	(TFT_HSYNC_PULSE + TFT_HSYNC_BACK_PORCH + TFT_WIDTH  + TFT_HSYNC_FRONT_PORCH)
#define	TFT_VSYNC_PERIOD	(TFT_VSYNC_PULSE + TFT_VSYNC_BACK_PORCH + TFT_HEIGHT + TFT_VSYNC_FRONT_PORCH)
#define TFT_PCLK	(TFT_HSYNC_PERIOD * TFT_VSYNC_PERIOD * TFT_FPS)
#define LCD_FPR		((TFT_PCLK * 1048576)/100000000)

//=============================================================================
// SSD1963 commands
//=============================================================================
#define SSD1963_NOP						0x00
#define SSD1963_SOFT_RESET  			0x01
#define SSD1963_GET_POWER_MODE 			0x0A
#define SSD1963_GET_ADDRESS_MODE		0x0B
#define SSD1963_GET_DISPLAY_MODE		0x0D
#define SSD1963_GET_TEAR_EFFECT_STATUS 	0x0E
#define SSD1963_ENTER_SLEEP_MODE		0x10
#define SSD1963_EXIT_SLEEP_MODE			0x11
#define SSD1963_ENTER_PARTIAL_MODE		0x12
#define SSD1963_ENTER_NORMAL_MODE		0x13
#define SSD1963_EXIT_INVERT_MODE		0x20
#define SSD1963_ENTER_INVERT_MODE		0x21
#define SSD1963_SET_GAMMA_CURVE			0x26
#define SSD1963_SET_DISPLAY_OFF			0x28
#define SSD1963_SET_DISPLAY_ON			0x29
#define SSD1963_SET_COLUMN_ADDRESS		0x2A
#define SSD1963_SET_PAGE_ADDRESS		0x2B
#define SSD1963_WRITE_MEMORY_START		0x2C
#define SSD1963_READ_MEMORY_START		0x2E
#define SSD1963_SET_PARTIAL_AREA		0x30
#define SSD1963_SET_SCROLL_AREA			0x33
#define SSD1963_SET_TEAR_OFF			0x34
#define SSD1963_SET_REAR_ON				0x35
#define SSD1963_SET_ADDRESS_MODE		0x36
#define SSD1963_SET_SCROLL_START		0x37
#define SSD1963_EXIT_IDLE_MODE			0x38
#define SSD1963_ENTER_IDLE_MODE			0x39
#define SSD1963_SET_PIXEL_FORMAT		0x3A
#define SSD1963_WRITE_MEMORY_CONTINUE	0x3C
#define SSD1963_READ_MEMORY_CONTINUE	0x3E
#define SSD1963_SET_TEAR_SCANLINE		0x44
#define SSD1963_GET_SCANLINE			0x45
#define SSD1963_READ_DDB				0xA1
#define SSD1963_SET_LCD_MODE			0xB0
#define SSD1963_GET_LCD_MODE			0xB1
#define SSD1963_SET_HORI_PERIOD			0xB4
#define SSD1963_GET_HORI_PERIOD			0xB5
#define SSD1963_SET_VERT_PERIOD			0xB6
#define SSD1963_GET_VERT_PERIOD			0xB7
#define SSD1963_SET_GPIO_CONF			0xB8
#define SSD1963_GET_GPIO_CONF			0xB9
#define SSD1963_SET_GPIO_VALUE			0xBA
#define SSD1963_GET_GPIO_STATUS			0xBB
#define SSD1963_SET_POST_PROC			0xBC
#define SSD1963_GET_POST_PROC			0xBD
#define SSD1963_SET_PWM_CONF			0xBE
#define SSD1963_GET_PWM_CONF			0xBF
#define SSD1963_GET_LCD_GEN0			0xC0
#define SSD1963_SET_LCD_GEN0			0xC1
#define SSD1963_GET_LCD_GEN1			0xC2
#define SSD1963_SET_LCD_GEN1			0xC3
#define SSD1963_GET_LCD_GEN2			0xC4
#define SSD1963_SET_LCD_GEN2			0xC5
#define SSD1963_GET_LCD_GEN3			0xC6
#define SSD1963_SET_LCD_GEN3			0xC7
#define SSD1963_SET_GPIO0_ROP			0xC8
#define SSD1963_GET_GPIO0_ROP			0xC9
#define SSD1963_SET_GPIO1_ROP			0xCA
#define SSD1963_GET_GPIO1_ROP			0xCB
#define SSD1963_SET_GPIO2_ROP			0xCC
#define SSD1963_GET_GPIO2_ROP			0xCD
#define SSD1963_SET_GPIO3_ROP			0xCE
#define SSD1963_GET_GPIO3_ROP			0xCF
#define SSD1963_SET_DBC_CONF			0xD0
#define SSD1963_GET_DBC_CONF			0xD1
#define SSD1963_SET_DBC_TH				0xD4
#define SSD1963_GET_DBC_TH				0xD5
#define SSD1963_SET_PLL					0xE0
#define SSD1963_SET_PLL_MN				0xE2
#define SSD1963_GET_PLL_MN				0xE3
#define SSD1963_GET_PLL_STATUS			0xE4
#define SSD1963_SET_DEEP_SLEEP			0xE5
#define SSD1963_SET_LSHIFT_FREQ			0xE6
#define SSD1963_GET_LSHIFT_FREQ			0xE7
#define SSD1963_SET_PIXEL_DATA_INTERFACE 0xF0
	#define SSD1963_PDI_8BIT			0
	#define SSD1963_PDI_12BIT			1
	#define SSD1963_PDI_16BIT			2
	#define SSD1963_PDI_16BIT565		3
	#define SSD1963_PDI_18BIT			4
	#define SSD1963_PDI_24BIT			5
	#define SSD1963_PDI_9BIT			6
#define SSD1963_GET_PIXEL_DATA_INTERFACE 0xF1

/*
//-----------------color----------------------------------
#define RED	  0XF800
#define GREEN 0xF81F //0X07E0
#define GREEN1 0x07C0
#define BLUE  0xFFE0 //0X001F  
#define BRED  0XF81F
#define GRED  0XFFE0
#define GBLUE 0X07FF
#define WHITE 0XFFFF 
#define BLACK 0X0000 
#define BROWN 0XBC40 
#define BRRED 0XFC07 
#define GRAY  0X8430 
#define LGRAY 0XC618 
#define YELOW 0x07FF
//----------------------------------------------------
*/


// Translates a 24-bit RGB color to RGB565
#define TRANSLATE24BIT_TO_RGB565(c)    ((((c) & 0x00ff0000) >> 19) |               \
                                 ((((c) & 0x0000ff00) >> 5) & 0x000007e0) | \
                                 ((((c) & 0x000000ff) << 8) & 0x0000f800))

// Define a basic set of 24bit colors, based on the standard "websafe" set
#define COLOR24_AQUA	0x00FFFF
#define COLOR24_GREY	0x808080
#define COLOR24_NAVY 	0x000080 	
#define COLOR24_SILVER 	0xC0C0C0
#define COLOR24_BLACK 	0x000000 	
#define COLOR24_GREEN 	0x008000 	
#define COLOR24_OLIVE 	0x808000 	
#define COLOR24_TEAL 	0x008080
#define COLOR24_BLUE 	0x0000FF 	
#define COLOR24_LIME 	0x00FF00 	
#define COLOR24_PURPLE 	0x800080 	
#define COLOR24_WHITE 	0xFFFFFF
#define COLOR24_FUCHSIA	0xFF00FF 	
#define COLOR24_MAROON	0x800000 	
#define COLOR24_RED 	0xFF0000
#define COLOR24_YELLOW 	0xFFFF00

// Create a set of RGB565 colors that can be used directly within code
#define COLOR_AQUA TRANSLATE24BIT_TO_RGB565(COLOR24_AQUA)
#define COLOR_GREY	TRANSLATE24BIT_TO_RGB565(COLOR24_GREY)
#define COLOR_NAVY	TRANSLATE24BIT_TO_RGB565(COLOR24_NAVY) 	
#define COLOR_SILVER 	TRANSLATE24BIT_TO_RGB565(COLOR24_SILVER)
#define COLOR_BLACK 	TRANSLATE24BIT_TO_RGB565(COLOR24_BLACK) 	
#define COLOR_GREEN 	TRANSLATE24BIT_TO_RGB565(COLOR24_GREEN) 	
#define COLOR_OLIVE 	TRANSLATE24BIT_TO_RGB565(COLOR24_OLIVE) 	
#define COLOR_TEAL 		TRANSLATE24BIT_TO_RGB565(COLOR24_TEAL)
#define COLOR_BLUE 		TRANSLATE24BIT_TO_RGB565(COLOR24_BLUE) 	
#define COLOR_LIME 		TRANSLATE24BIT_TO_RGB565(COLOR24_LIME) 	
#define COLOR_PURPLE 	TRANSLATE24BIT_TO_RGB565(COLOR24_PURPLE) 	
#define COLOR_WHITE 	TRANSLATE24BIT_TO_RGB565(COLOR24_WHITE)
#define COLOR_FUCHSIA	TRANSLATE24BIT_TO_RGB565(COLOR24_FUCHSIA) 	
#define COLOR_MAROON	TRANSLATE24BIT_TO_RGB565(COLOR24_MAROON)	
#define COLOR_RED 		TRANSLATE24BIT_TO_RGB565(COLOR24_RED)
#define COLOR_YELLOW 	TRANSLATE24BIT_TO_RGB565(COLOR24_YELLOW)

unsigned const short Char_Dot[760] =//744
{
   0x0000,0x0000,0x0000,0x001C,0x0020,0x0040,0x0040,0x0040,/*" No used*/
   0x0040,0x0040,0x0020,0x001C,0x0000,0x0000,0x0000,0x0000,/*# No used*/
   0x0000,0x0000,0x0000,0xE000,0x1000,0x0800,0x0800,0x0800,/*$ No used*/
   0x0C30,0x0C60,0x00C0,0x0180,0x0300,0x0630,0x0C30,0x0000,/*%    */
   0x0800,0x0800,0x1000,0xE000,0x0000,0x0000,0x0000,0x0000,/*& */
   0x2010,0x2010,0x2010,0x2010,0x2010,0x2010,0x3FF0,0x0000,/*' Empty bottom*/
   0x0000,0x0000,0x0FC0,0x1FE0,0x3030,0x2010,0x0000,0x0000,/*(    */
   0x0000,0x0000,0x2010,0x3030,0x1FE0,0x0FC0,0x0000,0x0000,/*)    */
   0x0000,0x18C0,0x0D80,0x0700,0x0700,0x0D80,0x18C0,0x0000,/**  */
   0x0000,0x0100,0x0100,0x07C0,0x07C0,0x0100,0x0100,0x0000,/*+    */
   0x0000,0x0000,0x0000,0xFFFC,0x0000,0x0000,0x0000,0x0000,/*, No used*/
   0x0000,0x0100,0x0100,0x0100,0x0100,0x0100,0x0100,0x0000,/*-    */
   0x0000,0x0000,0x0000,0x0030,0x0030,0x0000,0x0000,0x0000,/*.    */
   0x0030,0x0060,0x00C0,0x0180,0x0300,0x0600,0x0C00,0x0000,/*/    */
   0x1FE0,0x3FF0,0x2090,0x2310,0x2410,0x3FF0,0x1FE0,0x0000,/*0    */
   0x0000,0x0810,0x1810,0x3FF0,0x3FF0,0x0010,0x0010,0x0000,/*1    */
   0x1070,0x30F0,0x2190,0x2310,0x2610,0x3830,0x1830,0x0000,/*2    */
   0x1020,0x3030,0x2210,0x2210,0x2210,0x3FF0,0x1DE0,0x0000,/*3    */
   0x0300,0x0700,0x0D00,0x1910,0x3FF0,0x3FF0,0x0110,0x0000,/*4    */
   0x3E20,0x3E30,0x2210,0x2210,0x2310,0x23F0,0x21E0,0x0000,/*5    */
   0x0FE0,0x1FF0,0x3210,0x2210,0x2210,0x03F0,0x01E0,0x0000,/*6    */
   0x3000,0x3000,0x20F0,0x21F0,0x2300,0x3E00,0x3C00,0x0000,/*7    */
   0x1DE0,0x3FF0,0x2210,0x2210,0x2210,0x3FF0,0x1DE0,0x0000,/*8    */
   0x1C00,0x3E10,0x2210,0x2210,0x2230,0x3FE0,0x1FC0,0x0000,/*9    */
   0x0000,0x0000,0x0000,0x0C60,0x0C60,0x0000,0x0000,0x0000,/*:    */
   0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,0x0040,/*; No used*/
   0x0000,0x0100,0x0380,0x06C0,0x0C60,0x1830,0x1010,0x0000,/*<    */
   0x0240,0x0240,0x0240,0x0240,0x0240,0x0240,0x0240,0x0000,/*=    */
   0x0000,0x1010,0x1830,0x0C60,0x06C0,0x0380,0x0100,0x0000,/*>    */
   0x1800,0x3800,0x2000,0x23B0,0x27B0,0x3C00,0x1800,0x0000,/*?    */
   0x0FC0,0x1020,0x2790,0x2490,0x27D0,0x1050,0x0F80,0x0000,/*@    */
   0x07F0,0x0FF0,0x1900,0x3100,0x1900,0x0FF0,0x07F0,0x0000,/*A    */
   0x2010,0x3FF0,0x3FF0,0x2210,0x2210,0x3FF0,0x1DE0,0x0000,/*B    */
   0x0FC0,0x1FE0,0x3030,0x2010,0x2010,0x3030,0x1860,0x0000,/*C    */
   0x2010,0x3FF0,0x3FF0,0x2010,0x3030,0x1FE0,0x0FC0,0x0000,/*D    */
   0x2010,0x3FF0,0x3FF0,0x2210,0x2710,0x3030,0x3870,0x0000,/*E    */
   0x2010,0x3FF0,0x3FF0,0x2210,0x2700,0x3000,0x3800,0x0000,/*F    */
   0x0FC0,0x1FE0,0x3030,0x2110,0x2110,0x31E0,0x19F0,0x0000,/*G    */
   0x3FF0,0x3FF0,0x0200,0x0200,0x0200,0x3FF0,0x3FF0,0x0000,/*H    */
   0x0000,0x0000,0x2010,0x3FF0,0x3FF0,0x2010,0x0000,0x0000,/*I    */
   0x00E0,0x00F0,0x0010,0x2010,0x3FF0,0x3FE0,0x2000,0x0000,/*J    */
   0x2010,0x3FF0,0x3FF0,0x0300,0x0F00,0x3CF0,0x3070,0x0000,/*K    */
   0x2010,0x3FF0,0x3FF0,0x2010,0x0010,0x0030,0x0030,0x0000,/*L    */
   0x3FF0,0x3FF0,0x1C00,0x0E00,0x1C00,0x3FF0,0x3FF0,0x0000,/*M    */
   0x3FF0,0x3FF0,0x1C00,0x0E00,0x0700,0x3FF0,0x3FF0,0x0000,/*N    */
   0x0FC0,0x1FE0,0x3030,0x2010,0x3030,0x1FE0,0x0FC0,0x0000,/*O    */
   0x2010,0x3FF0,0x3FF0,0x2210,0x2200,0x3E00,0x1C00,0x0000,/*P    */
   0x1FE0,0x3FF0,0x2010,0x2070,0x2030,0x3FF8,0x1FE8,0x0000,/*Q    */
   0x2010,0x3FF0,0x3FF0,0x2200,0x2300,0x3FF0,0x1CF0,0x0000,/*R    */
   0x1860,0x3C70,0x2610,0x2210,0x2310,0x39F0,0x18E0,0x0000,/*S    */
   0x0000,0x3800,0x3010,0x3FF0,0x3FF0,0x3010,0x3800,0x0000,/*T    */
   0x3FE0,0x3FF0,0x0010,0x0010,0x0010,0x3FF0,0x3FE0,0x0000,/*U    */
   0x3F80,0x3FC0,0x0060,0x0030,0x0060,0x3FC0,0x3F80,0x0000,/*V    */
   0x3FC0,0x3FF0,0x0070,0x01C0,0x0070,0x3FF0,0x3FC0,0x0000,/*W    */
   0x3030,0x3CF0,0x0FC0,0x0300,0x0FC0,0x3CF0,0x3030,0x0000,/*X    */
   0x0000,0x3C00,0x3E10,0x03F0,0x03F0,0x3E10,0x3C00,0x0000,/*Y    */
   0x3870,0x30F0,0x2190,0x2310,0x2610,0x3C30,0x3870,0x0000,/*Z    */
   0x0010,0x0030,0x0050,0x0190,0x0310,0x0C10,0x3FF0,0x0000,/*[ Delta*/
   0x0800,0x0800,0x0800,0x0800,0x0800,0x0800,0x0800,0x0800,/*\ No used*/
   0x0000,0x0000,0x0000,0xFC7C,0x0280,0x0280,0x0100,0x0100,/*] No used*/
   0x0200,0x0400,0x0C00,0x1FF0,0x3FF0,0x0C00,0x0400,0x0200,/*^ rising edge*/
   0x0100,0x0080,0x00C0,0x3FE0,0x3FF0,0x00C0,0x0080,0x0100,/*_ falling edge*/
   0x2010,0x2010,0x2010,0x2010,0x2010,0x2010,0x2010,0x2010,/*` Empty body*/
   0x00E0,0x05F0,0x0510,0x0510,0x07E0,0x03F0,0x0010,0x0000,/*a    */
   0x2010,0x3FF0,0x3FE0,0x0210,0x0210,0x03F0,0x01E0,0x0000,/*b    */
   0x03E0,0x07F0,0x0410,0x0410,0x0410,0x0630,0x0220,0x0000,/*c    */
   0x01E0,0x03F0,0x0610,0x2410,0x3FF0,0x3FF0,0x0010,0x0000,/*d    */
   0x03E0,0x07F0,0x0510,0x0510,0x0510,0x0730,0x0320,0x0000,/*e    */
   0x0000,0x0210,0x1FF0,0x3FF0,0x2210,0x3000,0x1800,0x0000,/*f    */
   0x03C8,0x07EC,0x0424,0x0424,0x03FC,0x07F8,0x0400,0x0000,/*g    */
   0x2010,0x3FF0,0x3FF0,0x0220,0x0400,0x07F0,0x03F0,0x0000,/*h    */
   0x0000,0x0000,0x0410,0x37F0,0x37F0,0x0010,0x0000,0x0000,/*i    */
   0x0000,0x0018,0x001C,0x0404,0x37FC,0x37F8,0x0400,0x0000,/*j    */
   0x2010,0x3FF0,0x3FF0,0x2180,0x03C0,0x0670,0x0430,0x0000,/*k    */
   0x0000,0x0000,0x2010,0x3FF0,0x3FF0,0x0010,0x0000,0x0000,/*l    */
   0x07F0,0x07F0,0x0600,0x03F0,0x0600,0x07F0,0x07F0,0x0000,/*m    */
   0x0400,0x07F0,0x03F0,0x0400,0x0400,0x07F0,0x03F0,0x0000,/*n    */
   0x03E0,0x07F0,0x0410,0x0410,0x0410,0x07F0,0x03E0,0x0000,/*o    */
   0x0404,0x07FC,0x03FC,0x0444,0x0440,0x07C0,0x0380,0x0000,/*p    */
   0x0380,0x07C0,0x0440,0x0444,0x07FC,0x03FC,0x0404,0x0000,/*q    */
   0x0410,0x07F0,0x03F0,0x0610,0x0400,0x0600,0x0300,0x0000,/*r    */
   0x0000,0x0220,0x0730,0x0590,0x0490,0x06F0,0x0260,0x0000,/*s    */
   0x0400,0x0400,0x1FE0,0x3FF0,0x0410,0x0430,0x0020,0x0000,/*t    */
   0x07E0,0x07F0,0x0010,0x0010,0x07E0,0x07F0,0x0010,0x0000,/*u    */
   0x0000,0x07C0,0x07E0,0x0030,0x0030,0x07E0,0x07C0,0x0000,/*v    */
   0x07E0,0x07F0,0x0030,0x00E0,0x0030,0x07F0,0x07E0,0x0000,/*w    */
   0x0410,0x0630,0x03E0,0x01C0,0x03E0,0x0630,0x0410,0x0000,/*x    */
   0x0780,0x07C4,0x0044,0x0044,0x0044,0x07F8,0x07F0,0x0000,/*y    */
   0x0630,0x0670,0x04D0,0x0590,0x0710,0x0630,0x0430,0x0000,/*z    */
   0x0780,0x0480,0x3CF0,0x2010,0x2FD0,0x2FD0,0x2FD0,0x2FD0,/*{ Battery head*/
   0x2FD0,0x2FD0,0x2FD0,0x2FD0,0x2FD0,0x2FD0,0x2FD0,0x2FD0,/*| Battery body*/
   0x2FD0,0x2FD0,0x2FD0,0x2FD0,0x2FD0,0x2010,0x3FF0,0x0000,/*} Battery bottom*/
   0x0780,0x0480,0x3CF0,0x2010,0x2010,0x2010,0x2010,0x2010 /*~ Empty head*/
};            


//=============================================================================
// Write command to SSD1963
//=============================================================================
void SSD1963_WriteCommand(unsigned int commandToWrite)
{
	volatile unsigned int dly;
	SSD1963_DATAPORT->ODR  = commandToWrite;
	SSD1963_CTRLPORT->BSRR = SSD1963_PIN_RD;
	SSD1963_CTRLPORT->BRR  = SSD1963_PIN_A0 | SSD1963_PIN_CS | SSD1963_PIN_WR;
	//for(dly = 0; dly < 1; dly++);//asm("nop");
	SSD1963_CTRLPORT->BSRR = SSD1963_PIN_A0 | SSD1963_PIN_CS | SSD1963_PIN_WR;
}
//=============================================================================
// Write data to SSD1963
//=============================================================================
void SSD1963_WriteData(unsigned long dataToWrite)
{
	volatile unsigned int dly;
	SSD1963_DATAPORT->ODR  = (dataToWrite & 0x00FFFF);											//GPIOB
	SSD1963_DATAPORT_1->ODR = (dataToWrite & 0xFF0000)>>16;		//GPIOC
	SSD1963_CTRLPORT->BSRR = SSD1963_PIN_RD | SSD1963_PIN_A0;
	SSD1963_CTRLPORT->BRR  = SSD1963_PIN_CS | SSD1963_PIN_WR;
	//for(dly = 0; dly < 1; dly++);//asm("nop");
	SSD1963_CTRLPORT->BSRR = SSD1963_PIN_CS | SSD1963_PIN_WR;
}
//=============================================================================
// 
//=============================================================================
void SSD1963_Init (void)
{
	volatile unsigned int dly;
	SSD1963_CTRLPORT->BRR = SSD1963_PIN_RST;
	for(dly = 0; dly < 1000; dly++);
	SSD1963_CTRLPORT->BSRR = SSD1963_PIN_RST;
	for(dly = 0; dly < 1000; dly++);

	SSD1963_WriteCommand(SSD1963_SOFT_RESET);
	
	// REFclk = crystal = 10MHz
	SSD1963_WriteCommand(SSD1963_SET_PLL_MN);
	SSD1963_WriteData(0x1D);	// PLLclk = REFclk * 30 (300MHz)
	SSD1963_WriteData(0x02);	// SYSclk = PLLclk / 3  (100MHz)
	SSD1963_WriteData(0x54);  // dummy

	SSD1963_WriteCommand(SSD1963_SET_PLL);
	SSD1963_WriteData(0x01);
 
	for(dly = 0; dly < 100000; dly++);

	SSD1963_WriteCommand(SSD1963_SET_PLL);
	SSD1963_WriteData(0x03);
	
	SSD1963_WriteCommand(0x01);  // soft reset
	
	//Set LSHIFT freq, i.e. the DCLK with PLL freq 120MHz set previously
	//Typical DCLK for TY430TFT480272 is 9MHz
	//9MHz = 100MHz*(LCDC_FPR+1)/2^20
	//LCDC_FPR = 94370 (0x170A2)
	SSD1963_WriteCommand(SSD1963_SET_LSHIFT_FREQ); 
	SSD1963_WriteData(0x01);
	SSD1963_WriteData(0x70);
	SSD1963_WriteData(0xA2);

  //Set panel mode, varies from individual manufacturer
	SSD1963_WriteCommand(SSD1963_SET_LCD_MODE); 
	SSD1963_WriteData(0x2C);//24-Bit //(0x0C); //18-Bit			
	SSD1963_WriteData(0x00);			
	SSD1963_WriteData((TFT_WIDTH-1)>>8);
	SSD1963_WriteData((TFT_WIDTH-1) & 0xFF);
	SSD1963_WriteData((TFT_HEIGHT-1)>>8);
	SSD1963_WriteData((TFT_HEIGHT-1) & 0xFF);
	SSD1963_WriteData(0x00);
	
	//Set horizontal period
	SSD1963_WriteCommand(SSD1963_SET_HORI_PERIOD);
	SSD1963_WriteData((TFT_HSYNC_PERIOD-1)>>8);
	SSD1963_WriteData((TFT_HSYNC_PERIOD-1) & 0xFF);
	SSD1963_WriteData((TFT_HSYNC_PULSE + TFT_HSYNC_BACK_PORCH-1)>>8);
	SSD1963_WriteData((TFT_HSYNC_PULSE + TFT_HSYNC_BACK_PORCH) & 0xFF);
	SSD1963_WriteData(TFT_HSYNC_PULSE-1);
	SSD1963_WriteData(0x00);			
	SSD1963_WriteData(0x00);
	SSD1963_WriteData(0x00);

	//Set vertical period
	SSD1963_WriteCommand(SSD1963_SET_VERT_PERIOD); 		
	SSD1963_WriteData((TFT_VSYNC_PERIOD-1)>>8);
	SSD1963_WriteData((TFT_VSYNC_PERIOD-1) & 0xFF);
	SSD1963_WriteData((TFT_VSYNC_PULSE + TFT_VSYNC_BACK_PORCH-1)>>8);
	SSD1963_WriteData((TFT_VSYNC_PULSE + TFT_VSYNC_BACK_PORCH-1) & 0xFF);
	SSD1963_WriteData(TFT_VSYNC_PULSE-1);
	SSD1963_WriteData(0x00);			
	SSD1963_WriteData(0x00);
	
	//Set pixel data interface
	SSD1963_WriteCommand(SSD1963_SET_PIXEL_DATA_INTERFACE);
	SSD1963_WriteData(SSD1963_PDI_24BIT);
	
	// set address mode
	SSD1963_WriteCommand(SSD1963_SET_ADDRESS_MODE);
	SSD1963_WriteData(0x45);

  // setting PWM for LED backlight
	SSD1963_WriteCommand(SSD1963_SET_PWM_CONF);
	SSD1963_WriteData(0x0E);
	SSD1963_WriteData(0xFF);   // Set 0xFF for full and 0x00 for off
	SSD1963_WriteData(0x01);
	SSD1963_WriteData(0x00);
	SSD1963_WriteData(0x00);
	SSD1963_WriteData(0x00);
	
	// GPIO0,1,2,3 is output
	SSD1963_WriteCommand(SSD1963_SET_GPIO_CONF);
	SSD1963_WriteData(0x0F);
	SSD1963_WriteData(0x01);
	
	// set GPIO0 , 1 , 2, 3 = 1
	SSD1963_WriteCommand(SSD1963_SET_GPIO_VALUE);
	SSD1963_WriteData(0x0F);
	
	//SET display on
	SSD1963_WriteCommand(SSD1963_SET_DISPLAY_ON);		
	
	// exit sleep mode
	SSD1963_WriteCommand(SSD1963_EXIT_SLEEP_MODE);
}
//=============================================================================
//
//=============================================================================
void SSD1963_SetArea(unsigned int sx, unsigned int ex, unsigned int sy, unsigned int ey)
{
	SSD1963_WriteCommand(SSD1963_SET_COLUMN_ADDRESS);	
	SSD1963_WriteData((sx >> 8) & 0xFF);
	SSD1963_WriteData((sx >> 0) & 0xFF);
	SSD1963_WriteData((ex >> 8) & 0xFF);
	SSD1963_WriteData((ex >> 0) & 0xFF);

	SSD1963_WriteCommand(SSD1963_SET_PAGE_ADDRESS);	
	SSD1963_WriteData((sy >> 8) & 0xFF);
	SSD1963_WriteData((sy >> 0) & 0xFF);
	SSD1963_WriteData((ey >> 8) & 0xFF);
	SSD1963_WriteData((ey >> 0) & 0xFF);
}
//=============================================================================
// Fill area of specified color
//=============================================================================
/*void SSD1963_FillArea(unsigned int sx, unsigned int ex, unsigned int sy, unsigned int ey, int color)
{
int i;
SSD1963_SetArea(sx, ex, sy, ey);
SSD1963_WriteCommand(SSD1963_WRITE_MEMORY_START);
for(i = 0; i < ((ex-sx+1)*(ey-sy+1)); i++)
	{
	SendData(color);
	}
}*/
//=============================================================================
// Fills whole screen specified color
//=============================================================================
void SSD1963_ClearScreen(unsigned long color)
{
	unsigned int x,y;
	SSD1963_SetArea(0, TFT_WIDTH-1 , 0, TFT_HEIGHT-1);
	SSD1963_WriteCommand(0x2c);
	for(x=0;x<TFT_WIDTH;x++)
		{
					for(y= 0;y<TFT_HEIGHT;y++)
									{
										SSD1963_WriteData(color);
									}
		}
}
//=============================================================================
//
//=============================================================================
void GLCD_SetPixel(int x, int y, unsigned long color)
{
	SSD1963_SetArea(x, x, y, y);
	SSD1963_WriteCommand(0x2c);
	SSD1963_WriteData(color);
	//SSD1963_WriteCommand(0x0);
}
//=============================================================================
/*******************************************************************************
 Display_Str: Display String   Input: XY, Color, Mode, String  
*******************************************************************************/
void Display_Str(unsigned short x0, unsigned short y0, unsigned long Color, unsigned const char *s)
{ 
  signed short i, j, b,ii=0; 
  while (*s!=0) {
    unsigned const short *scanline=Char_Dot+((*s-0x22)*8);
    for(i=0;i<8;++i){
      if((*s==0x20)||(*s==0x21)) b=0x0000;
      else b=scanline[i];
      if((*s==0x21)&&(i==4)) break;
      for(j=0;j<14;++j){
        if(b&4) {
          GLCD_SetPixel(x0+i, y0+j,Color);
        } else {
          GLCD_SetPixel(x0+i, y0+j,COLOR_BLACK);
        }
        b>>=1;
				
      }
			++ii;
    }
    if(*s==0x21) x0 +=4;       
    else  x0 += 8;             
    ++s;                           
  }
}

